/*
 * =============================================================================
 * Simplified BSD License, see http://www.opensource.org/licenses/
 * -----------------------------------------------------------------------------
 * Copyright (c) 2008-2009, Marco Terzer, Zurich, Switzerland
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 *     * Redistributions of source code must retain the above copyright notice, 
 *       this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright 
 *       notice, this list of conditions and the following disclaimer in the 
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Swiss Federal Institute of Technology Zurich 
 *       nor the names of its contributors may be used to endorse or promote 
 *       products derived from this software without specific prior written 
 *       permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE.
 * =============================================================================
 */
package ch.javasoft.jsmat.variable;

import java.io.DataOutput;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import ch.javasoft.jsmat.common.MatClass;

public class MatCell extends MatAllocated {
	
	private final Collection<? extends MatAllocated> mElements;
	
	public MatCell(String[] elements) {
		this(elements, new int[] {1, elements.length});
	}
	public MatCell(String[] elements, int[] dims) {
		this(createMatrixCollection(elements), dims);
	}
	public MatCell(Collection<? extends MatAllocated> elements) {
		this(elements, new int[] {1, elements.size()});
	}
	public MatCell(Collection<? extends MatAllocated> elements, int[] dims) {
		super(MatClass.CELL, dims);
		mElements = elements;
		checkValueLength(elements.size());
	}

	private static Collection<? extends MatAllocated> createMatrixCollection(String[] elements) {
		List<MatCharMatrix> list = new ArrayList<MatCharMatrix>(elements.length);
		for (int i = 0; i < elements.length; i++) {
			list.add(new MatCharMatrix(elements[i]));
		}
		return list;
	}

	@Override
	protected int getRawDataSize() {
		int size = 0;
		for (MatAllocated mx : mElements) {
			size += mx.getSize(MatVariable.DEFAULT_NAME);
		}
		return size;
	}

	@Override
	protected void writeRawData(DataOutput out) throws IOException {
		for (MatAllocated mx : mElements) {
			mx.write(MatVariable.DEFAULT_NAME, out);
		}
	}

}
